# -*- coding: utf-8 -*-
from sys import modules as sysmodules
from pkgutil import walk_packages
from importlib import import_module
from flask import Flask, request
from diandao.flask import SQLAlchemy
import time
import redis


DEFAULT_APP_NAME = "idea"

_vars = {"app": None, "config": {}, "db": None}

# 保存的数据库会话
DB_CONNECTIONS = {}


def create_app(config, app_name=None):
    if app_name is None:
        app_name = DEFAULT_APP_NAME

    app = Flask(app_name)
    if config is None:
        return app
    _vars['app'] = app

    configure_app(app, config)
    configure_db(app)
    configure_redis(app)
    configure_models(app)
    configure_blueprints(app)
    # configure_session(app)
    # configure_auth_handler(app)
    # configure_handler(app)
    # configure_filters(app)
    # #configure_jinja(app)
    # configure_rq(app)
    # configure_sentry(app)


    return app


# ------------
# 应用配置文件
def configure_app(app, config=None):
    # config 不为空时覆盖配置
    if config is not None:
        app.config.from_object(config)

    # 映射配置
    _vars['config'] = app.config


def configure_db(app):
    """
    数据库连接初始化

    :config SQLALCHEMY_POOL_RECYCLE: 连接池生命周期
    :config SQLALCHEMY_POOL_TIMEOUT: 连接池超时
    :config SQLALCHEMY_BINDS: 数据库连接配置, 可配置多个key=>value的配置

    :param app:
    :return:
    """
    from sqlalchemy.ext.declarative import declarative_base
    from sqlalchemy import event, exc, select
    db = SQLAlchemy(app)
    _vars['db'] = db

    @app.teardown_request
    def remove_session(exception=None):
        pass
        #pass
        #db.session.remove()
        #if exception and db.session.is_active:
            #db.session.rollback()


def configure_redis(app):
    """
    Redis 配置, 需要configs.py中的Redis参数

    :config REDIS: (host, port, database)

    :param app:
    :return:
    """
    host, port, db = app.config['REDIS']
    client = redis.Redis(host=host, port=port, db=db)
    app.redisClient = client


# 注册蓝图(Route)
def configure_blueprints(app):
    @app.before_request
    def before_request():
        request.timestamp = time.time()

    import blueprints
    bkeys = []
    prints = []
    for loader, name, is_pkg in walk_packages(blueprints.__path__):
        import_module(blueprints.__name__ + '.' + name)

    for name, blue in blueprints.__dict__.items():
        if hasattr(blue, "blueprint"):
            prints.append((blue.blueprint, getattr(blue, "url_prefix", None)))

    for blue, url_prefix in prints:
        app.register_blueprint(blue, url_prefix=url_prefix)


def configure_models(app):
    import models
    for loader, name, is_pkg in walk_packages(models.__path__):
        import_module(models.__name__ + '.' + name)
        module = getattr(models, name)
        model = getattr(module, name, None)
        if model is None:
            raise ImportError("class %s not found in %s" % (name, module.__file__))
        setattr(models, name, model)

# 获取app配置
def get_config():
    return _vars['config']


# 获取app配置
def get_app():
    return _vars['app']


def get_db():
    return _vars['db']


# 多数据库时, 取到指定数据库连接
def get_db_engine(bind):
    db = get_db()
    app = get_app()
    return db.get_engine(app, bind)


def db_connections():
    return DB_CONNECTIONS


# 初始化数据库
def init_db():
    """
    数据库初始化
    - 未创建表的模型会自动创建数据表
    - 已存在的表, 不会自动更新字段信息

    :return:
    """
    db = get_db()
    db.create_all()






# -*- coding: utf-8 -*-

import json as pjson
from json import JSONEncoder
from datetime import datetime, time

class JSON(JSONEncoder):

    def default(self, o):
        if hasattr(o, '__json__') and callable(getattr(o, '__json__')):
            return o.__json__()

        if hasattr(o, '__dict__'):
            return o.__dict__

        if isinstance(o, datetime):
            return o.strftime("%Y-%m-%d %H:%M:%S")

        if isinstance(o, time):
            return o.strftime("%H:%M:%S")

        raise TypeError(repr(o) + " is not JSON serializable")

    @classmethod
    def stringify(cls, o, **kws):
        return pjson.dumps(o, cls=cls, ensure_ascii=False, **kws)

    @classmethod
    def parse(cls, str, **kws):
        return pjson.loads(str, **kws)

    @classmethod
    def dict(cls, o, **kws):
        return cls.parse(cls.stringify(o, **kws))





# -*- coding: utf-8 -*-

from functools import partial
from werkzeug.local import LocalStack, LocalProxy

stacker = {"app": None}

def _lookup_(name):
    return stacker[name]

app=LocalProxy(partial(_lookup_,"app"))


# -*- coding: utf-8 -*-

from flask_sqlalchemy import SQLAlchemy as Flask_SQLAlchemy
from .globals import *
from flask import request
from sqlalchemy import event
from sqlalchemy.orm import mapper
from sqlalchemy.inspection import inspect
from diandao import isint_or


class SQLAlchemy(Flask_SQLAlchemy):
    def __init__(self, app=None, use_native_unicode=True, session_options=None, metadata=None):
        """重构父级初始化方法,缓存app对象"""
        super(self.__class__, self).__init__(app, use_native_unicode, session_options, metadata)
        if app is not None:
            apply_app(app)
        pass

    def init_app(self, app):
        """重构父级初始化方法,缓存app对象"""
        super(self.__class__, self).init_app(app)
        if app is not None:
            apply_app(app)
        pass


def apply_app(app):
    stacker["app"] = app
    app.config.setdefault("CACHE_LIFE_TIME", 600)


class BaseInterface(object):
    """基础接口类"""
    # 绑定指定模型类, 可继承模型的类方法(@classmethod)
    __bindmodel__ = None

    def __getattr__(self, name):
        if self.__bindmodel__ is not None:
            return getattr(self.__bindmodel__, name, None)
        return None


class version_control:
    def __getattribute__(self, name):  # real signature unknown; restored from __doc__
        """ x.__getattribute__('name') <==> x.name """
        pass

    def __get__(self, obj, type=None):  # real signature unknown; restored from __doc__
        """ descr.__get__(obj[, type]) -> value """
        pass

    def __set__(self, obj, type=None):  # real signature unknown; restored from __doc__
        """ descr.__get__(obj[, type]) -> value """
        pass

    def __del__(self, obj, type=None):  # real signature unknown; restored from __doc__
        """ descr.__get__(obj[, type]) -> value """
        pass

    def __init__(self, function):  # real signature unknown; restored from __doc__
        pass

    __func__ = property(__get__, __set__, __del__)  # default


def version(ver):
    """版本号修饰"""
    return version_control

def operator_role():
    """请求头消息中获取操作者数据"""
    return  request.headers.get("Operator-Role", "")

def operator_id():
    """请求头消息中获取操作者数据"""
    return  request.headers.get("Operator-Id", 0)

def instant_defaults_listener(target, args, kwargs):
    """
    模型初始化时, 字段使用默认值
    """
    for key, column in inspect(target.__class__).columns.items():
        # if key in ["creatorid", "operatorid"]:
        #     # 请求头消息中获取操作者数据, 自动填入creatorid, operatorid字段
        #     operatorid = operator_id()
        #     if operatorid is not None:
        #         setattr(target, key, operatorid)
        #         continue
        # # operator role
        # if key in ["creator_role", "operator_role"]:
        #     # 请求头消息中获取操作者数据, 自动填入creatorid, operatorid字段
        #     role = operator_role()
        #     if operatorrole is not None:
        #         setattr(target, key, role)
        #         continue

        if column.default is not None:
            if callable(column.default.arg):
                setattr(target, key, column.default.arg(target))
            else:
                setattr(target, key, column.default.arg)

# 默认值初始化
event.listen(mapper, 'init', instant_defaults_listener)


from setuptools import setup

setup(
    name='diandao',
    version="0.2.0",
    url='https://git.oschina.net/diandao/pylib',
    license='MIT',
    author='idollo',
    author_email='stone58@qq.com',
    description='diandao python libs.',
    long_description=__doc__,
    packages=['diandao', 'diandao.flask'],
    include_package_data=True,
    zip_safe=False,
    platforms='any',
    install_requires=[
        'Flask==0.10.1',
        'Flask-Script==2.0.5',
        'SQLAlchemy==1.0.9',
        'redis>=2.10.5',
        'hiredis==0.2.0'
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5'
    ]
)



# -*- coding: utf-8 -*-
import sys
import pkgutil
import importlib

# 可转化字符串类型的数字为int
def isint_or(var, replace=0):
    """
    判断是否int, 否则使用replace替换
    :param var:
    :param replace:
    :return:
    """
    if isinstance(var, int) or isinstance(var, long):
        return var
    if isinstance(var, basestring) and var.isnumeric():
        return int(var)
    return replace



# 拼接数据为字符串
# 当把一组ID用逗号分割拼接为字符串时, 使用 str.join()会报错
# 故需要一个拼接方法, 自适应数据类型拼接成字符串
def implode(sep, items):
    """
    拼接数据为字符串
    :param sep: 间隔字符
    :param items: 数据元组
    :return: string, 拼接后的字符串

    nums = [1,2,3,4,5]
    nums = implode(":", nums) // >> "1:2:3:4:5"

    """
    return str(sep).join(map(__stringify, items))

def splitids(str):
    """
    分离id串为int数组
    :para str: id串
    :return:   list(ids)
    """
    str = str.strip()
    if not str:
        return []
    return map(lambda id: int(id), str.split(","))
    

def __stringify(v):
    """
    implode 转化数据为字符串回调方法
    """
    if v is None: return ''
    if isinstance(v, basestring) and len(v) == 0: return ''
    if isinstance(v, basestring): return v
    return str(v)


def import_submodules(package_name):
    """ Import all submodules of a module, recursively

    :param package_name: Package name
    :type package_name: str
    :rtype: dict[types.ModuleType]
    """
    package = sys.modules[package_name]
    return {
        name: importlib.import_module(package_name + '.' + name)
        for loader, name, is_pkg in pkgutil.walk_packages(package.__path__)
    }
